//
//  Main
//
//  Created by Shawn on 2007-10-17.
//

package
{
    import fl.controls.*;
    import flash.display.*;
    import flash.events.*;
    import flash.text.*;
    import de.polygonal.ds.*;
    import edu.harvard.med.nano.*;
    import com.yahoo.astra.utils.*;
    import flash.geom.Point;
    
    public class Main extends Sprite {
        // constants, colors, etc.
        public static const ROWS:int = 8;
        public static const COLS:int = 8;
        public static const SLICEBG:int = 0xffffff; //0xcccccc;
        public static const PATHBG:int = 0xffffff;
        public static const RENDERBG:int = 0xffffff;
        public static const BASEBG:int = 0xffcc99;
        
        // default variable parameters
        public var radius:int = 20;
        public var pathScale:int = 1;
        
        // Display components
        private var slicePanel:Sprite;
        private var sliceMask:MovieClip;
        private var sliceBorder:Sprite;
        private var pathPanel:Sprite;
        private var pathMask:MovieClip;
        private var pathBorder:Sprite;
        private var renderPanel:Sprite;
        private var renderMask:MovieClip;
        private var renderBorder:Sprite;
        private var basePanel:Sprite;
        private var baseText:TextField;
        
        // Buttons
        private var zoomInSlice:Button;
        private var zoomOutSlice:Button;
        private var nextSlice:Button;
        private var prevSlice:Button;
        private var zoomInPath:Button;
        private var zoomOutPath:Button;
        
        // Data containers
        private var sliceList:DLinkedList;
        private var sliceIter:DListIterator;
        private var drawSlice:DrawSlice;
        private var path:Path;
        private var drawPath:DrawPath;
        
        // Misc
        private var pathPanelCenterPoint:Point;
        //private var circ:Sprite;
        
        public function Main() {
            // adjust root position
            this.x = this.y = 20;
            
            // set up main sprites and buttons
            setupSlicePanel();
            setupPathPanel();
            setupRenderPanel();
            setupBasePanel();
            setupBaseText();
            setupButtons();
            
            // set up data structures and drawing routines
            setupPath();
            setupSlice();
            
/*            circ = new Sprite();
            circ.graphics.beginFill(BASEBG);
            circ.graphics.drawCircle(490,258,5);
            circ.graphics.endFill();
            addChild(circ);
*/        
        }
        
        private function setupSlicePanel():void {
            // panel
            this.slicePanel = new Sprite();
            this.slicePanel.graphics.beginFill(SLICEBG);
            this.slicePanel.graphics.drawRoundRect(0,0,320,516,10);
            this.slicePanel.graphics.endFill();
            this.addChild(slicePanel);
            // mask
            this.sliceMask = new MovieClip();
            this.sliceMask.graphics.beginFill(0xffffff);
            this.sliceMask.graphics.drawRoundRect(0,0,320,516,10);
            this.sliceMask.graphics.endFill();
            this.slicePanel.addChild(sliceMask);
            this.slicePanel.mask = sliceMask;
            // border
            this.sliceBorder = new Sprite();
            this.sliceBorder.graphics.lineStyle(1, 0xcc6600, 1, true);
            this.sliceBorder.graphics.drawRoundRect(0,0,320,516,10);
            this.sliceBorder.graphics.endFill();
            this.addChild(sliceBorder);
        }
        
        private function setupPathPanel():void {
            // panel
            this.pathPanel = new Sprite();
            this.pathPanel.x = 330;
            this.pathPanel.graphics.beginFill(PATHBG);
            this.pathPanel.graphics.drawRoundRect(0,0,320,516,10);
            this.pathPanel.graphics.endFill();
            this.addChild(pathPanel);
            // mask
/*          this.pathMask = new MovieClip();
            this.pathMask.x = 330;
            this.pathMask.graphics.beginFill(0xffffff);
            this.pathMask.graphics.drawRoundRect(0,0,320,516,10);
            this.pathMask.graphics.endFill();
            this.pathPanel.addChild(pathMask);
            this.pathPanel.mask = pathMask;
*/            // border
            this.pathBorder = new Sprite();
            this.pathBorder.x = 330;
            this.pathBorder.graphics.lineStyle(1, 0x0066cc, 1, true);
            this.pathBorder.graphics.drawRoundRect(0,0,320,516,10);
            this.pathBorder.graphics.endFill();
            this.addChild(pathBorder);
            
            trace(this.pathPanel.x,this.pathPanel.y);
            trace(this.pathPanel.width, this.pathPanel.height);
            this.pathPanelCenterPoint = new Point(this.pathPanel.width/2,0);
        }
        
        private function setupRenderPanel():void {
            // panel
            this.renderPanel = new Sprite();
            this.renderPanel.x = 660;
            this.renderPanel.graphics.beginFill(RENDERBG);
            this.renderPanel.graphics.drawRoundRect(0,0,320,516,10);
            this.renderPanel.graphics.endFill();
            this.addChild(renderPanel);
            // mask
/*          this.renderMask = new MovieClip();
            this.renderPanel.x = 660;
            this.renderMask.graphics.beginFill(0xffffff);
            this.renderMask.graphics.drawRoundRect(0,0,320,516,10);
            this.renderMask.graphics.endFill();
            this.renderPanel.addChild(renderMask);
            this.renderPanel.mask = renderMask;
*/            // border
            this.renderBorder = new Sprite();
            this.renderBorder.x = 660;
            this.renderBorder.graphics.lineStyle(1, 0x006633, 1, true);
            this.renderBorder.graphics.drawRoundRect(0,0,320,516,10);
            this.renderBorder.graphics.endFill();
            this.addChild(renderBorder);
        }
        
        private function setupBasePanel():void {
            this.basePanel = new Sprite();
            this.basePanel.y = 560;
            this.basePanel.graphics.beginFill(BASEBG);
            this.renderPanel.graphics.lineStyle(1, 0, 1, true);
            this.basePanel.graphics.drawRoundRect(0,0,320,100,10);
            this.basePanel.graphics.endFill();
            this.addChild(basePanel);
        }
        
        private function setupBaseText():void {
            this.baseText = new TextField()
            this.basePanel.addChild(baseText);
            this.baseText.x = 10;
            this.baseText.y = 10;
            this.baseText.width = 300;
            this.baseText.height = 90;
            this.baseText.text = "foo";
        }
        
        private function setupButtons():void {
            /* Slice */
            zoomInSlice = new Button();
            zoomOutSlice = new Button();
            nextSlice = new Button();
            prevSlice = new Button();
            // labels
            zoomInSlice.label = "in";
            zoomOutSlice.label = "out";
            nextSlice.label = "+";
            prevSlice.label = "-";
            // size & position
            zoomInSlice.width = zoomOutSlice.width = nextSlice.width = prevSlice.width = 30;
            zoomInSlice.height = zoomOutSlice.height = nextSlice.height = prevSlice.height = 25;
            zoomInSlice.move(15, 520);
            zoomOutSlice.move(45, 520);
            nextSlice.move(100, 520);
            prevSlice.move(130, 520);
            this.addChild(zoomInSlice);
            this.addChild(zoomOutSlice);
            this.addChild(nextSlice);
            this.addChild(prevSlice);
            // event handlers
            zoomInSlice.addEventListener(MouseEvent.CLICK, zoomInSliceAction);
            zoomOutSlice.addEventListener(MouseEvent.CLICK, zoomOutSliceAction);
            nextSlice.addEventListener(MouseEvent.CLICK, nextSliceAction);
            prevSlice.addEventListener(MouseEvent.CLICK, prevSliceAction);
            
            /* Path */
            zoomInPath = new Button();
            zoomOutPath = new Button();
            // labels
            zoomInPath.label = "in";
            zoomOutPath.label = "out";
            // size & position
            zoomInPath.width = zoomOutPath.width = 30;
            zoomInPath.height = zoomOutPath.height = 25;
            zoomInPath.move(315, 520);
            zoomOutPath.move(345, 520);
            this.addChild(zoomInPath);
            this.addChild(zoomOutPath);
            // event handlers
            zoomInPath.addEventListener(MouseEvent.CLICK, zoomInPathAction);
            zoomOutPath.addEventListener(MouseEvent.CLICK, zoomOutPathAction);
        }
        
        private function zoomInSliceAction(event:MouseEvent):void {
            if (radius < 50) {
                radius = radius + 5;
            }
            drawSlice.update(sliceIter.data, radius);
        }
        
        private function zoomOutSliceAction(event:MouseEvent):void {
            if (radius > 5) {
                radius = radius - 5;
            }
            drawSlice.update(sliceIter.data, radius);
        }
        
        private function nextSliceAction(event:MouseEvent):void {
            sliceIter.forth();
            var num:int = sliceIter.data.sliceNum;
            drawPath.currentSlice = num;
            if (sliceIter.atTail()) {
                num = num + 1
                sliceList.append(new Slice(ROWS,COLS, num));
                path.rightSliceIndex = num;
                path.appendSegment(num);
            }
            drawSlice.update(sliceIter.data, radius);
            drawPath.update();
        }
        
        private function prevSliceAction(event:MouseEvent):void {
            sliceIter.back();
            var num:int = sliceIter.data.sliceNum;
            drawPath.currentSlice = num;
            if (sliceIter.atHead()) {
                num = num - 1;
                sliceList.prepend(new Slice(ROWS,COLS, num));
                path.leftSliceIndex = num;
                path.prependSegment(num);
            }
            drawSlice.update(sliceIter.data, radius);
            drawPath.update();
        }
        
        private function zoomInPathAction(event:MouseEvent):void {
            this.pathPanel.scaleX = this.pathPanel.scaleY = NumberUtil.roundToPrecision(this.pathPanel.scaleX,1);
            trace(this.pathPanel.scaleX);
            if (this.pathPanel.scaleX < 0.5) {
                DynamicRegistration.scale(this.pathPanel, this.pathPanelCenterPoint, this.pathPanel.scaleX + 0.1, this.pathPanel.scaleY + 0.1)
            } else {
                DynamicRegistration.scale(this.pathPanel, this.pathPanelCenterPoint, this.pathPanel.scaleX + 0.5, this.pathPanel.scaleY + 0.5)
            }
            drawPath.update();
        }
        
        private function zoomOutPathAction(event:MouseEvent):void {
            this.pathPanel.scaleX = this.pathPanel.scaleY = NumberUtil.roundToPrecision(this.pathPanel.scaleX,1);
            trace(this.pathPanel.scaleX);
            if (this.pathPanel.scaleX > 0.5) {
                DynamicRegistration.scale(this.pathPanel, this.pathPanelCenterPoint, this.pathPanel.scaleX - 0.5, this.pathPanel.scaleY - 0.5)
            } else if (this.pathPanel.scaleX > 0.1) {
                DynamicRegistration.scale(this.pathPanel, this.pathPanelCenterPoint, this.pathPanel.scaleX - 0.1, this.pathPanel.scaleY - 0.1)
            }
            drawPath.update();
        }
        
        /**
         * Set up Slice drawing routines
         **/
        private function setupPath():void {
            // Path data and drawing routines
            path = new Path();
            drawPath = new DrawPath(path);
            this.pathPanel.addChild(drawPath);
        }
        
        /**
         * Set up Slice data structure and drawing routine
         **/
        private function setupSlice():void {
            sliceList = new DLinkedList;
            // Instantiate linked list of three Slices
            for (var i:int = -2; i < 2; i++) {
                sliceList.append(new Slice(ROWS,COLS, i));
            }
            path.leftSliceIndex = -2;
            path.rightSliceIndex = 1;
            
            // set up Iterator for traversing the sliceList
            sliceIter = sliceList.getListIterator();
            sliceIter.forth(); // move to center slice
            sliceIter.forth();
            drawSlice = new DrawSlice(ROWS,COLS,radius, path, drawPath);
            drawSlice.x = drawSlice.y = 40;
            this.slicePanel.addChild(drawSlice);
            drawSlice.update(sliceIter.data, radius);
        }
    }
}