//
//  DrawNode
//
//  Created by Shawn on 2007-11-05.
//  Copyright (c) 2007 Harvard University. All rights reserved.
//

/*
DrawNode contains code related to drawing the individual circles
representing helices in the slice panel.
*/

package edu.harvard.med.nano
{

    import flash.display.*;
    import flash.events.*;
    import flash.text.*;
    import edu.harvard.med.nano.SliceNode;
    
    class DrawNode extends Sprite {
        // Color values
        public static var COLOR_FILL:uint = 0xeeeeee;//0xffffff;
        public static var COLOR_BLANK:uint = 0x999999;
        public static var COLOR_ROLLOVER:uint = 0x0066cc;
        public static var COLOR_ROLLFILL:uint = 0x99ccff;
        public static var COLOR_MARK:uint = 0xcc6600; //0xf40095;
        public static var COLOR_MARKFILL:uint = 0xffcc99; //0xf40095;
        
        // SliceNode-cloned variables
        public var sliceNode:SliceNode;     // the sliceNode
        
        // DrawNode-specific variables
        public var radius:int;
        public var row:int;
        public var col:int;
        public var tf:TextField;
        public var tfm:TextFormat;
        // coordinates for hover distance calculations
        public var p0x,p0y,p1x,p1y,p2x,p2y:int;
        public var number:int = -1;
        
        public function DrawNode(row:int, col:int, radius:int=20)
        {
            // Minimal setup; everything else is added on first refreshNode()
            this.row = row;
            this.col = col;
            this.radius = radius;
            tf = new TextField();
            tfm = tf.getTextFormat();
            tf.antiAliasType = "advanced";
            tf.autoSize = "left";
            tf.selectable = false;
            tf.mouseEnabled = false;
            this.addChild(tf);
        }
        
        public function refreshNode():void
        {
            /**
             * Node coordinate determination
             * Note: in Flash, the stage y direction is reversed from Cartesian y.
             **/
            this.x = this.col*this.radius*1.732051;
            if ((row % 2) ^ (col % 2)) {  // odd parity
                this.y = this.row*this.radius*3 + this.radius;
                this.p0x = this.radius*-0.866; // odd p0 = 150ª -> (-0.866, -0.5)
                this.p0y = this.radius*-0.5;
                this.p1x = this.radius*0.866;  // odd p1 = 30ª -> (0.866, -0.5)
                this.p1y = this.radius*-0.5;
                this.p2x = this.radius*0;      // odd p2 = 270ª -> (0, 1)
                this.p2y = this.radius*1;
            } else {                      // even parity
                this.y = this.row*this.radius*3;
                this.p0x = this.radius*0.866;  // even p0 = 330ª -> (0.866, 0.5)
                this.p0y = this.radius*0.5;
                this.p1x = this.radius*-0.866; // even p1 = 210ª -> (-0.866, 0.5)
                this.p1y = this.radius*0.5;
                this.p2x = this.radius*0;      // even p2 = 90ª -> (0, -1)
                this.p2y = this.radius*-1;
            }
            
            /**
             * Node drawing
             **/
            var color:uint;
            var fill:uint;
            graphics.clear();
            if (this.sliceNode.marked) {
                color = COLOR_MARK;
                fill = COLOR_MARKFILL;
                // always draw colored nodes on top
                this.parent.setChildIndex(this, this.parent.numChildren-1);
            } else {
                color = COLOR_BLANK;
                fill = COLOR_FILL;
            }
            graphics.beginFill(fill, 1);
            graphics.lineStyle(1, color);
            graphics.drawCircle(0, 0, radius);
            graphics.endFill();
            
            /**
             * Text format and coordinate determination
             **/
            if (this.sliceNode.number != -1 && this.sliceNode.marked) {
                this.tf.text = this.sliceNode.number.toString();
            } else {
                this.tf.text = "";
            }
            this.tfm.size = this.radius;
            this.tf.setTextFormat(this.tfm);
            var offset:int; // set offset according to number of digits
            if (this.tf.text.length == 1) {
                offset = 4;
            } else if (this.tf.text.length == 2) {
                offset = -3;
            } else {
                offset = -1 - this.tf.text.length;
            }
            this.tf.x = (0 - this.radius/2) + offset;
            this.tf.y = (0 - this.radius/2);
        }
        
        /**
         * Returns all neighbors, ranked by distance from mouse pointer.
         * Parameters can be set to exclude marked and/or unmarked nodes.
         **/
        public function getNeighbors(mouseX:int, mouseY:int, getMarked:Boolean=true, getUnmarked:Boolean=true):Array {
            var neighbors:Array = new Array();
            
            // Calculate distance to neighbors
            if (this.sliceNode.p0neighbor != null) {
                this.sliceNode.p0neighbor.dist = getDist(this.p0x, mouseX, this.p0y, mouseY)
                if ((this.sliceNode.p0neighbor.marked && getMarked) ||
                    (!this.sliceNode.p0neighbor.marked && getUnmarked)) {
                        neighbors.push(this.sliceNode.p0neighbor);
                }
            }
            if (this.sliceNode.p1neighbor != null) {
                this.sliceNode.p1neighbor.dist = getDist(this.p1x, mouseX, this.p1y, mouseY)
                if ((this.sliceNode.p1neighbor.marked && getMarked) ||
                    (!this.sliceNode.p1neighbor.marked && getUnmarked)) {
                        neighbors.push(this.sliceNode.p1neighbor);
                }
            }
            if (this.sliceNode.p2neighbor != null) {
                this.sliceNode.p2neighbor.dist = getDist(this.p2x, mouseX, this.p2y, mouseY)
                if ((this.sliceNode.p2neighbor.marked && getMarked) || 
                    (!this.sliceNode.p2neighbor.marked && getUnmarked)) {
                        neighbors.push(this.sliceNode.p2neighbor);
                }
            }
            neighbors.sortOn("dist", Array.NUMERIC);
            return neighbors;
        }
        
        public function hover(p:int):void
        {
            var color:uint;
            var fill:uint;
            graphics.clear();
            if (this.sliceNode.marked) {
                color = COLOR_MARK;
                fill = COLOR_FILL;
            } else {
                color = COLOR_ROLLOVER;
                fill = COLOR_ROLLFILL;
            }
            graphics.beginFill(fill, 1);
            graphics.lineStyle(1, color);
            graphics.drawCircle(0, 0, this.radius);
            graphics.endFill();
            
            // If current node is marked, show we can unmark it
            if (this.sliceNode.marked) {
                //trace("marked");
            } else {
                graphics.beginFill(COLOR_ROLLOVER, 1);
                //graphics.lineStyle(1, COLOR_ROLLOVER);
                if (p == 0) {
                    graphics.moveTo(0, 0)
                    graphics.lineTo(this.p0x, this.p0y);
                } else if (p == 1) {
                    graphics.moveTo(0, 0)
                    graphics.lineTo(this.p1x, this.p1y);
                } else {
                    graphics.moveTo(0, 0)
                    graphics.lineTo(this.p2x, this.p2y);
                }
                graphics.endFill();
            }
            
            // always draw colored nodes on top
            this.parent.setChildIndex(this, this.parent.numChildren-1);
        }
        
        private function getDist(x1:int,x2:int,y1:int,y2:int):int {
            return Math.sqrt(Math.pow(x1-x2,2)+Math.pow(y1-y2,2));
        }
        
        public function unhover():void
        {
            var color:uint;
            var fill:uint;
            graphics.clear();
            if (this.sliceNode.marked) {
                color = COLOR_MARK;
                fill = COLOR_MARKFILL;
                // always draw colored nodes on top
                this.parent.setChildIndex(this, this.parent.numChildren-1);
            } else {
                color = COLOR_BLANK;
                fill = COLOR_FILL;
            }
            graphics.beginFill(fill, 1);
            graphics.lineStyle(1, color);
            graphics.drawCircle(0, 0, radius);
            graphics.endFill();
        }
    }
}