//
//  DrawSegment
//
//  Created by Shawn on 2007-11-11.
//

/*
DrawSegment handles the segment-level drawing of 
the scaffold and staple paths.

One DrawSegment is used for each 21-base stretch of the DNA structure.
An instance of DrawPath is used to create and refresh DrawSegments.
*/

package edu.harvard.med.nano
{
    import flash.display.*;
    import flash.events.*;
    import flash.text.*;
    import edu.harvard.med.nano.PathSegment;
    
    class DrawSegment extends Sprite {
        public static var COLOR_SCAF:uint = 0x999999;
        public static var COLOR_STAP:uint = 0x990000;
        public static var COLOR_GRID:uint = 0xeeeeee;
        public static var COLOR_7:uint = 0xcccccc;
        
        // Pointer to data
        public var pathSegment:PathSegment;
        
        // DrawSegment-specific variables
        public var scale:int = 1;
        private var drawHeight:int = 10;
/*        public var tf:TextField;
        public var tfm:TextFormat;
*/        
        function DrawSegment(x:int=0, y:int=0, scale:int=1) {
            // Minimal setup; everything else is added on first refreshNode()
            this.x = x;
            this.y = y;
            //this.scale = scale;
            /*
            this.tf = new TextField();
            this.tfm = tf.getTextFormat();
            tf.antiAliasType = "advanced";
            tf.autoSize = "left";
            tf.selectable = false;
            tf.mouseEnabled = false;
            this.addChild(tf);*/
        }
        
        public function refreshSegment(scale:int=1) {
            //this.scale = scale;
            this.refreshPositions();
            this.refreshXovers();
        }
        
        private function refreshPositions() {
            var i:int;
            graphics.clear();
            
            // draw a vertical line to represent each segment boundary
            graphics.beginFill(0xffffff,0);
            
            // highlight current slice (NEED TO FIX)
/*            if (currentSlice == segIter.node.data.segIndex) {
                // highlight the current slice
                graphics.lineStyle(1, 0xcc0000, 1, true);
            } else {
                graphics.lineStyle(1, 0x0066cc, 1, true);
            }
*/
            graphics.lineStyle(1, 0x0066cc, 1, true);
            graphics.moveTo(this.x, this.y);
            graphics.lineTo(this.x, this.y+this.drawHeight);
            graphics.endFill();

            
            // draw a tick mark to represent each base position
            for (i = 0; i < 21; i++) {
                graphics.beginFill(0xffcc99, 1);
                if (i % 7 == 0) {
                    graphics.lineStyle(1, COLOR_7);
                } else {
                    graphics.lineStyle(1, COLOR_GRID);
                }
                graphics.moveTo(this.x + i*2, this.y + (drawHeight/2)-2);
                graphics.lineTo(this.x + i*2, this.y + (drawHeight/2)+2);
                graphics.endFill();
            }
            
            
        }
        
        private function refreshXovers() {
            var i:int;
            var xoverColor:uint;
            
            for (i = 0; i < 21; i++) {
                if (this.pathSegment.scafxover[i] != -1) {
                    if (this.pathSegment.helixNum % 2 == 1) {
                        xoverColor = 0x00cc00; // green
                    } else {
                        xoverColor = 0x00cccc; // cyan
                    }
                    
                    if (i == 4 || i == 15) { // p0
                        graphics.beginFill(0xffffff, 1);
                        graphics.lineStyle(1, xoverColor);
                        graphics.moveTo(this.x+i*2, this.y);
                        graphics.lineTo(this.x+i*2, this.pathSegment.p0.drawSegment.y);
                        graphics.endFill();
                    } else if (i == 8 || i == 18) { // p1
                        graphics.beginFill(0xffffff, 1);
                        graphics.lineStyle(1, xoverColor);
                        graphics.moveTo(this.x+i*2, this.y);
                        graphics.lineTo(this.x+i*2, this.pathSegment.p1.drawSegment.y);
                        graphics.endFill();
                    } else if (i == 1 || i == 11) { // p2
                        graphics.beginFill(0xffffff, 1);
                        graphics.lineStyle(1, xoverColor);
                        graphics.moveTo(this.x+i*2, this.y);
                        graphics.lineTo(this.x+i*2, this.pathSegment.p2.drawSegment.y);
                        graphics.endFill();
                    }
                }
            }
        }
    }
}