//
//  DrawSlice
//
//  Created by Shawn on 2007-10-18.
//

package edu.harvard.med.nano
{

    import flash.display.*;
    import flash.events.*;
    import flash.text.*;
    import edu.harvard.med.nano.*;
    import de.polygonal.ds.LinkedStack;
    
    public class DrawSlice extends Sprite {
        
        // Bookkeeping
        private var rows:int;
        private var cols:int;
        private var drawNodeArray:Object = new Object();
        private var nodeNumbers:LinkedStack = new LinkedStack();
        private var nodeCount:int = 0;
        private var path:Path;
        private var drawPath:DrawPath;
        
        function DrawSlice(rows:int, cols:int, radius:int, path:Path, drawPath:DrawPath) {
            var node:DrawNode;
            var index:String;
            
            this.rows = rows;
            this.cols = cols;
            this.path = path;
            this.drawPath = drawPath;
            
            for (var col:int = 0; col < cols; col++)
            {
                for (var row:int = 0; row < rows; row++)
                {
                    // create node and setup handlers
                    node = new DrawNode(row, col, radius);
                    addChild(node);
                    node.addEventListener(MouseEvent.MOUSE_MOVE, onMove, false);
                    node.addEventListener(MouseEvent.MOUSE_OUT,  onMouseOut, false);
                    node.addEventListener(MouseEvent.CLICK, onClick, false);
                    
                    // add new node to the hash
                    index = row + "," + col;
                    drawNodeArray[index] = node;
                }
            }
        }
        
        // update nodes with new data
        public function update(slice: Slice, radius:int):void
        {
            var sliceHash:Object = slice.sliceHash;
            var index:String;
            var sliceNode:SliceNode;
            var drawNode:DrawNode;
            
            // loop through sliceList positions, and update the status
            // for corresponding node positions
            for (var col:int = 0; col < this.cols; col++)
            {
                for (var row:int = 0; row < this.rows; row++)
                {
                    index = row + "," + col;
                    sliceNode = sliceHash[index];
                    drawNode = drawNodeArray[index];
                    drawNode.sliceNode = sliceNode; // point at sliceNode
                    sliceNode.drawNode = drawNode;
                    sliceNode.number = drawNode.number;
                    drawNode.radius = radius;
                    drawNode.refreshNode();
                }
            }
        }
        
        private function onMove(e:MouseEvent):void
        {
            var p:int;
            
            // if marked, hover marked neighbor
            if (e.target.sliceNode.marked == true) {
                p = e.target.sliceNode.pairedPosition;
                e.target.hover(p);
                e.target.sliceNode.pairedNeighbor.drawNode.hover(p);
            } else {
                // if not marked, hover nearest unmarked neighbor
                var neighbors:Array = e.target.getNeighbors(e.localX, e.localY, false, true);
                if (neighbors.length == 0) {
                    return;
                }
                var nearest:SliceNode = neighbors[0];
                var index:String = nearest.row + "," + nearest.col;
                if (e.target.sliceNode.p0neighbor == nearest) {
                    p = 0;
                } else if (e.target.sliceNode.p1neighbor == nearest) {
                    p = 1;
                } else {
                    p = 2;
                }
                e.target.hover(p);
                drawNodeArray[index].hover(p);
                neighbors.splice(0,1); // remove nearest from list
                var neighbor:SliceNode;
                for (var i:int = 0; i < neighbors.length; i++)
                {
                    neighbor = neighbors[i];
                    index = neighbor.row + "," + neighbor.col;
                    drawNodeArray[index].unhover();
                }
            }
        }
        
        private function onMouseOut(e:MouseEvent):void
        {
            var neighbors:Array = e.target.getNeighbors(e.localX, e.localY);
            var neighbor:SliceNode;
            var index:String;
            
            for (var i:int = 0; i < neighbors.length; i++)
            {
                neighbor = neighbors[i];
                index = neighbor.row + "," + neighbor.col;
                drawNodeArray[index].unhover();
            }
            e.target.unhover();
            
        }
        
        /**
         * onClick handles the toggling of node pairing, assigning of numbers
         * Marked nodes will be unmarked along with their neighbors
         * Unmarked nodes will be marked and numbered
         **/
        private function onClick(e:MouseEvent):void
        {
            var node:SliceNode = e.target.sliceNode;
            var neighbors:Array; // will be used to retrieve neighbors
            var markedNeighbors:Array;
            
            // first check if a marked node was clicked
            if (node.marked) {
                // unmark it and its paired neighbor
                node.marked = false;
                node.pairedNeighbor.marked = false;
                // unpair both
                node.pairedNeighbor.pairedNeighbor = null;
                node.pairedNeighbor = null;
                
                // adjust Path
                
                
            } else if (!node.marked) {
                // get unmarked neighbors only
                neighbors = e.target.getNeighbors(e.localX, e.localY,false,true);
                if (neighbors.length == 0) {
                    return;  // no unmarked neighbors, clicking does nothing
                } else {
                    // get the nearby neighbor that was hovering
                    var nearest:SliceNode = neighbors[0];
                    
                    // NOTE: should probably set up a separate mechanism
                    // to handle number assignment
                    var odd,even:int;
                    // get some new numbers to assign
                    if (e.target.number == -1 || nearest.drawNode.number == -1) {
                        if (nodeNumbers.peek() == null) {
                            nodeNumbers.push(nodeCount);
                            nodeCount += 1;
                            nodeNumbers.push(nodeCount);
                            nodeCount += 1;
                        }
                        odd = nodeNumbers.pop();
                        even = nodeNumbers.pop();
                    }
                    // mark and number nodes
                    if (node.parity == 1) {
                        node.marked = true;
                        nearest.marked = true;
                        // if this position has never been marked
                        if (e.target.number == -1) {
                            e.target.number = odd;
                            node.number = odd;
                            path.addHelix(odd); // notify path of new helix
                        } else {
                            path.extendHelix(odd);
                        }
                        if (nearest.drawNode.number == -1) {
                            nearest.drawNode.number = even;
                            nearest.number = even;
                            path.addHelix(even); // notify path of new helix
                        } else {
                            path.extendHelix(even);
                        }
                    } else {
                        node.marked = true;
                        nearest.marked = true;
                        if (e.target.number == -1) {
                            e.target.number = even;
                            node.number = even;
                            path.addHelix(even); // notify path of new helix
                        } else {
                            path.extendHelix(even);
                        }
                        if (nearest.drawNode.number == -1) {
                            nearest.drawNode.number = odd;
                            nearest.number = odd;
                            path.addHelix(odd); // notify path of new helix
                        } else {
                            path.extendHelix(odd);
                        }
                    }
                    
                    // pair nodes
                    node.pairedNeighbor = nearest;
                    nearest.pairedNeighbor = node;
                    
                    // store crossover position for future hovering
                    if (nearest == node.p0neighbor) {
                        node.pairedPosition = 0;
                        nearest.pairedPosition = 0;
                    } else if (nearest == node.p1neighbor) {
                        node.pairedPosition = 1;
                        nearest.pairedPosition = 1;
                    } else {
                        node.pairedPosition = 2;
                        nearest.pairedPosition = 2;
                    }
                    
                    // install pointers to marked neighbors in PathSegment
                    // get a list of marked neighbors
                    // set up pointer in PathSegment
                    if (node.p0neighbor != null) {
                        if (node.p0neighbor.marked) {
                            path.linkPathSegments(node.number, node.p0neighbor.number, 0);
                        }
                    }
                    if (node.p1neighbor != null) {
                        if (node.p1neighbor.marked) {
                            path.linkPathSegments(node.number, node.p1neighbor.number, 1);
                        }
                    }
                    if (node.p2neighbor != null) {
                        if (node.p2neighbor.marked) {
                            path.linkPathSegments(node.number, node.p2neighbor.number, 2);
                        }
                    }
                    if (nearest.p0neighbor != null) {
                        if (nearest.p0neighbor.marked) {
                            path.linkPathSegments(nearest.number, nearest.p0neighbor.number, 0);
                        }
                    }
                    if (nearest.p1neighbor != null) {
                        if (nearest.p1neighbor.marked) {
                            path.linkPathSegments(nearest.number, nearest.p1neighbor.number, 1);
                        }
                    }
                    if (nearest.p2neighbor != null) {
                        if (nearest.p2neighbor.marked) {
                            path.linkPathSegments(nearest.number, nearest.p2neighbor.number, 2);
                        }
                    }
                    path.pairHelices(drawPath.currentSlice,even,odd,nearest.pairedPosition);
                    path.drawPath.update();
                }
            }
            e.target.refreshNode();
            neighbors = e.target.getNeighbors(e.localX, e.localY);
            var neighbor:SliceNode;
            var index:String;
            for (var i:int = 0; i < neighbors.length; i++)
            {
                neighbor = neighbors[i];
                index = neighbor.row + "," + neighbor.col;
                drawNodeArray[index].refreshNode();
            }
        }
    }
}
