//
//  Slice
//
//  Created by Shawn on 2007-10-17.
//

/*
A Slice instance is meant to store a 2D array of helix data, in the form
of SliceNodes.
*/

package edu.harvard.med.nano
{
    import edu.harvard.med.nano.SliceNode;
    
    public class Slice {
        // store slice nodes in hash accessible by "row,col" keys
        public var sliceHash:Object;
        public var sliceNum:int;
        
        public function Slice(rows:int, cols:int, num:int)
        {
            var index:String;
            this.sliceNum = num;
            this.sliceHash = new Object();
            
            // create a SliceNode at each position in the grid
            for (var col:int = 0; col < cols; col++)
            {
                for (var row:int = 0; row < rows; row++)
                {
                    index = row + "," + col;
                    this.sliceHash[index] = new SliceNode(row, col);
                }
            }
            
            // Populate neighbor linkages
            var parity:int;
            var node:SliceNode;
            for (col = 0; col < cols; col++)
            {
                for (row = 0; row < rows; row++)
                {
                    index = row + "," + col;
                    node = this.sliceHash[index];
                    
                    if (node.parity) // odd parity
                    {
                        // if col-1 exists, set P0
                        if (col > 0) {
                            index = row + "," + (col-1);
                            node.p0neighbor = this.sliceHash[index];
                        }
                        // if col+1 exists, set P1
                        if (col < cols-1) {
                            index = row + "," + (col+1);
                            node.p1neighbor = this.sliceHash[index];
                        }
                        // if row+1 exists, set P2
                        if (row < rows-1) {
                            index = (row+1) + "," + col;
                            node.p2neighbor = this.sliceHash[index];
                        }
                    } else {         // even parity
                        // if col+1 exists, set P0
                        if (col < cols-1) {
                            index = row + "," + (col+1);
                            node.p0neighbor = this.sliceHash[index];
                        }
                        // if col-1 exists, set P1
                        if (col > 0) {
                            index = row + "," + (col-1);
                            node.p1neighbor = this.sliceHash[index];
                        }
                        // if row-1 exists, set P2
                        if (row > 0) {
                            index = (row-1) + "," + col;
                            node.p2neighbor = this.sliceHash[index];
                        }
                    }
                }
            }
        }
    }
}
